/***
 * Create a template function.
 * Syntax for template is `{{property}}` or `{{ if (someCondition} { out.push(someProp) } }}`
 * @param {string} template Template string
 * @param {{dataName:string,outName:string}}? options Options for template
 * @returns {(data: any) => string} Template function
 */
export function template(template, _a) {
    var _b = _a === void 0 ? {} : _a, _c = _b.dataName, dataName = _c === void 0 ? '$__' : _c, _d = _b.outName, outName = _d === void 0 ? 'out' : _d;
    var re = /{{(.+?)}}/g;
    var reExp = /(^( )?(if|for|else|switch|case|break|{|}))(.*)?/g;
    var code = "with(" + dataName + ") {var " + outName + "=[];\n";
    var cursor = 0;
    var match;
    var add = function (line, js) {
        js ? (code += line.match(reExp) ? line + '\n' : "try{" + outName + ".push(" + line + ')}catch(_unused){};\n') : (code += line !== '' ? outName + ".push(\"" + line.replace(/"/g, '\\"') + '");\n' : '');
        return add;
    };
    // tslint:disable-next-line no-conditional-assignment
    while (match = re.exec(template)) {
        add(template.slice(cursor, match.index))(match[1], true);
        cursor = match.index + match[0].length;
    }
    add(template.substr(cursor, template.length - cursor));
    code += "return " + outName + ".join(\"\");}";
    return new Function(dataName, code.replace(/[\r\t\n]/g, ''));
}
//# sourceMappingURL=templates.js.map